/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.utils;

import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.util.utils.BigDecimalUtil;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.utils: DecimalCalcUtil.java, v 0.1 2019-11-17 下午19:34 zhenxuan.luo Exp $ 
 */
public class DecimalCalcUtil {
    
    public static <T> T findLimitKey(Map<T, BigDecimal> vals, boolean isMax) {
        T ret = null;
        BigDecimal markVal = null;
        
        for (T key : vals.keySet()) {
            BigDecimal curVal = vals.get(key);
            if (ret == null) {
                ret = key;
                markVal = curVal;
            } else if (isMax && BigDecimalUtil.isGT(curVal, markVal)) {
                ret = key;
                markVal = curVal;
            } else if (!isMax && BigDecimalUtil.isLT(curVal, markVal)) {
                ret = key;
                markVal = curVal;
            }
        }
        return ret;
    }
    
    public static <T> BigDecimal calcSplitFactor(Map<T, BigDecimal> res, Map<T, BigDecimal> dist,
                                                 boolean isAvgWhileZero) {
        
        if (dist == null || dist.size() == 0) {
            return BigDecimal.ONE;
        }
        
        List<SplitItem<T>> splitItems = Lists.newArrayList();
        BigDecimal totalDist = BigDecimal.ZERO;
        res.clear();
        BigDecimal ret = BigDecimal.ONE;
        for (T key : dist.keySet()) {
            SplitItem splitItem = new SplitItem<T>();
            
            BigDecimal val = dist.get(key);
            VarChecker.checkNonnegative(val, "数值分配待分配项中不能包含负数[{0}: {1}]", key, val);
            splitItem.key = key;
            splitItem.val = val;
            splitItems.add(splitItem);
            res.put(key, BigDecimal.ZERO);
            totalDist = totalDist.add(val);
        }
        
        return calcRatioAct(res, totalDist, splitItems, isAvgWhileZero);
    }
    
    public static <T> BigDecimal calcSplitFactorToOne(Map<T, BigDecimal> res, Map<T, BigDecimal> dist,
                                                      boolean isAvgWhileZero) {
        
        if (dist == null || dist.size() == 0) {
            return BigDecimal.ONE;
        }
        
        List<SplitItem<T>> splitItems = Lists.newArrayList();
        BigDecimal totalDist = BigDecimal.ZERO;
        for (T key : dist.keySet()) {
            SplitItem splitItem = new SplitItem<T>();
            
            BigDecimal val = dist.get(key);
            VarChecker.checkNonnegative(val, "数值分配待分配项中不能包含负数[{0}: {1}]", key, val);
            splitItem.key = key;
            splitItem.val = val;
            splitItems.add(splitItem);
            res.put(key, BigDecimal.ZERO);
            totalDist = totalDist.add(val);
        }
    
        return calcRatioToOne(res, totalDist, splitItems, isAvgWhileZero);
    }
    
    private static <T> BigDecimal calcRatioAct(Map<T, BigDecimal> res, BigDecimal totalDist,
                                               List<SplitItem<T>> splitItems,
                                               boolean isAvgWhileZero) {
        
        BigDecimal expectRatioSum = BigDecimal.ONE;
        if (BigDecimalUtil.isZero(totalDist)) {
            if (isAvgWhileZero) {
                // 所有值都是0，且允许均摊
                for (int i = 0; i < splitItems.size(); i++) {
                    SplitItem<T> item = splitItems.get(i);
                    BigDecimal factor = BigDecimal.ONE.divide(new BigDecimal(splitItems.size()),
                            AbssqrConstants.DECIMAL_SCALE, AbssqrConstants.DECIMAL_ROUNDING_MODE);
                    expectRatioSum = expectRatioSum.subtract(factor);
                    res.put(item.key, factor);
                }
            } else {
                throw new AbssqrBizException(
                        MessageUtil.formatMsg("数值分配，分母求和为0，且非均摊模式，无法分配[{0}]", splitItems),
                        CommonErrorCodeEnum.INVALID_INTERNAL_MODEL, false);
            }
        } else {
            Collections.sort(splitItems, Comparator.comparing(SplitItem::getVal));
            for (int i = splitItems.size() - 1; i >= 0; i--) {
                SplitItem<T> item = splitItems.get(i);
                BigDecimal factor = item.val.divide(totalDist, AbssqrConstants.DECIMAL_SCALE,
                        AbssqrConstants.DECIMAL_ROUNDING_MODE);
                expectRatioSum = expectRatioSum.subtract(factor);
                res.put(item.key, factor);
            }
        }
        // 直接返回计算后比例，不做凑1处理
        return expectRatioSum;
    }
    
    private static <T> BigDecimal calcRatioToOne(Map<T, BigDecimal> res, BigDecimal totalDist,
                                                         List<SplitItem<T>> splitItems,
                                                         boolean isAvgWhileZero) {
        SplitItem<T> lastItem = null;
        BigDecimal remain = BigDecimal.ONE;
        BigDecimal expectRatioSum = BigDecimal.ONE;
    
        if (BigDecimalUtil.isZero(totalDist)) {
            if (isAvgWhileZero) {
                // 所有值都是0，且允许均摊
                for (int i = 0; i < splitItems.size() - 1; i++) {
                    SplitItem<T> item = splitItems.get(i);
                    BigDecimal factor = BigDecimal.ONE.divide(new BigDecimal(splitItems.size()),
                            AbssqrConstants.DECIMAL_SCALE, AbssqrConstants.DECIMAL_ROUNDING_MODE);
                    factor = BigDecimalUtil.min(remain, factor);
                    remain = remain.subtract(factor);
                    expectRatioSum = expectRatioSum.subtract(factor);
                    res.put(item.key, factor);
                }
                lastItem = splitItems.get(splitItems.size() - 1);
            } else {
                throw new AbssqrBizException(
                        MessageUtil.formatMsg("数值分配，分母求和为0，且非均摊模式，无法分配[{0}]", splitItems),
                        CommonErrorCodeEnum.INVALID_INTERNAL_MODEL, false);
            }
        } else {
            Collections.sort(splitItems, Comparator.comparing(SplitItem::getVal));
            for (int i = splitItems.size() - 1; i >= 0; i--) {
                SplitItem<T> item = splitItems.get(i);
                
                if (BigDecimalUtil.isLTE(item.val, BigDecimal.ZERO)) {
                    break;
                } else if (lastItem != null) {
                    BigDecimal factor = lastItem.val.divide(totalDist, AbssqrConstants.DECIMAL_SCALE,
                            AbssqrConstants.DECIMAL_ROUNDING_MODE);
                    factor = BigDecimalUtil.min(remain, factor);
                    remain = remain.subtract(factor);
                    expectRatioSum = expectRatioSum.subtract(factor);
                    res.put(lastItem.key, factor);
                }
                lastItem = item;
            }
        }
        
        // 尾数，凑1处理
        expectRatioSum = expectRatioSum.subtract(remain);
        res.put(lastItem.key, remain);

        return expectRatioSum;
    }
    
    private static class SplitItem<T> extends BaseEntity {
        T          key;
        BigDecimal val;
        
        public BigDecimal getVal() {
            return val;
        }
    }
}