/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.util.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;

import com.general.system.common.util.StringUtil;
import org.springframework.util.CollectionUtils;

import static com.general.system.common.util.MessageUtil.formatMsg;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.util.utils: TextUtil.java, v 0.1 2019-05-28 11:39 zhenxuan.luo Exp $ 
 */
public class TextUtil {
    /**
     * 单元项之间的分隔符
     */
    public final static String ITEM_SEP = ",";
    
    /**
     * 值之间的分隔符，用于map
     */
    public final static String VALUE_SEP = ":";
    
    /**
     * 将Set转成String
     *
     * @param obj
     * @return
     */
    public final static String convertSet2Txt(Set<String> obj) {
        return convertSet2Txt(obj, ITEM_SEP);
    }
    
    /**
     * 将Set转成String
     *
     * @param obj
     * @return
     */
    public final static String convertSet2Txt(Set<String> obj, String seq) {
        StringBuilder builder = new StringBuilder();
        
        if (!CollectionUtils.isEmpty(obj)) {
            for (String txt : obj) {
                builder.append(txt).append(seq);
            }
            builder.deleteCharAt(builder.length() - 1);
        }
        
        return builder.toString();
    }
    
    /**
     * 将文本String转成Set，用“,”分隔
     *
     * @param txt
     * @return
     */
    public final static Set<String> convertTxt2Set(String txt) {
        
        return convertTxt2Set(txt, ITEM_SEP);
    }
    
    /**
     * 将文本String转成Set，用自定义分隔
     *
     * @param txt
     * @return
     */
    public final static Set<String> convertTxt2Set(String txt, String sep) {
        
        Set<String> res = new HashSet<String>();
        if (StringUtil.isEmpty(txt)) {
            return res;
        }
        
        String[] txtParts = txt.split(sep);
        
        for (String txtPart : txtParts) {
            if (!StringUtil.isEmpty(txtPart)) {
                res.add(txtPart.trim());
            }
        }
        
        return res;
    }
    
    /**
     * 将文本String转成List，用“,”分隔
     *
     * @param txt
     * @return
     */
    public final static List<String> convertTxt2List(String txt) {
        
        return convertTxt2List(txt, ITEM_SEP);
    }
    
    /**
     * 将文本String转成List，用自定义分隔
     *
     * @param txt
     * @return
     */
    public final static List<String> convertTxt2List(String txt, String sep) {
        
        List<String> res = new ArrayList<String>();
        if (StringUtil.isEmpty(txt)) {
            return res;
        }
        
        String[] txtParts = txt.split(sep);
        
        for (String txtPart : txtParts) {
            if (!StringUtil.isEmpty(txtPart)) {
                res.add(txtPart.trim());
            }
        }
        
        return res;
    }
    
    /**
     * 将map转为String
     *
     * @param obj
     * @return
     */
    public final static String convertMap2Txt(Map<String, ? extends Object> obj) {
        StringBuilder builder = new StringBuilder();
        
        if (!CollectionUtils.isEmpty(obj)) {
            for (String key : obj.keySet()) {
                builder.append(key).append(VALUE_SEP).append(String.valueOf(obj.get(key)))
                        .append(ITEM_SEP);
            }
            builder.deleteCharAt(builder.length() - 1);
        }
        
        return builder.toString();
    }
    
    /**
     * 将String转为map
     *
     * @param txt
     * @return
     */
    public final static Map<String, String> convertTxt2Map(String txt) {
        
        Map<String, String> res = new HashMap<String, String>();
        
        if (StringUtil.isEmpty(txt)) {
            return res;
        }
        
        String[] parts = txt.split(ITEM_SEP);
        
        for (String part : parts) {
            
            String[] valParts = part.split(VALUE_SEP, 2);
            
            if (valParts == null || valParts.length != 2) {
                throw new RuntimeException(formatMsg("将{0}转成map失败，存在非法格式串{1}", txt, part));
            }
            
            if (valParts[0] == null || valParts[1] == null) {
                continue;
            }
            res.put(valParts[0].trim(), valParts[1].trim());
        }
        
        return res;
    }
    
    /**
     * 解析Y、N
     *
     * @param txt
     * @param defaultVal
     * @return
     */
    public static boolean parseYesOrNo(String txt, boolean defaultVal) {
        
        if (null == txt) {
            return defaultVal;
        }
        
        if (StringUtil.equalsIgnoreCase(AbssqrConstants.YES_FLAG, txt.trim())) {
            return true;
        } else if (StringUtil.equalsIgnoreCase(AbssqrConstants.NO_FLAG, txt.trim())) {
            return false;
        } else {
            return defaultVal;
        }
    }
    
    /**
     * 转换true false为Y或者N
     *
     * @param val
     * @return
     */
    public static String convertYesOrNo(boolean val) {
        return val ? AbssqrConstants.YES_FLAG : AbssqrConstants.NO_FLAG;
    }
    
    /**
     * 复制map
     *
     * @param orig
     * @return
     */
    public static Map<String, String> copyMap(Map<String, String> orig) {
        
        Map<String, String> newMap = new HashMap<String, String>();
        
        if (null != orig) {
            
            for (String key : orig.keySet()) {
                newMap.put(key, orig.get(key));
            }
        }
        
        return newMap;
    }
    
    /**
     * 合并两个map
     *
     * @param baseMap 基础map，如果有key和mergeMap冲突，使用baseMap中的值
     * @param mergeMap
     * @return
     */
    public static Map<String, String> mergeMap(Map<String, String> baseMap,
                                               Map<String, String> mergeMap) {
        // 1.先取被合并的map
        Map<String, String> newMap = copyMap(mergeMap);
        
        // 2.直接将baseMap中的值装入
        for (String key : baseMap.keySet()) {
            newMap.put(key, baseMap.get(key));
        }
        return newMap;
    }
}